/*

  dogm128.h

  (c) 2010 Oliver Kraus (olikraus@gmail.com)
  
  This file is part of the dogm128 Arduino library.

  The dogm128 Arduino library is free software: you can redistribute it and/or modify
  it under the terms of the Lesser GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  The dogm128 Arduino library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  Lesser GNU General Public License for more details.

  You should have received a copy of the Lesser GNU General Public License
  along with dogm128.  If not, see <http://www.gnu.org/licenses/>.

  one of the following constants could be defined:
    DOGM128_HW
    DOGS102_HW
    DOGM132_HW
*/

#ifndef _DOGM128_H
#define _DOGM128_H

#define DOG_REVERSE

#define DOGM128_HW	/* uncomment for the DOGM128 display */
//#define DOGS102_HW		/* uncomment for the DOGS102 display */
//#define DOGM132_HW	/* uncomment for the DOGM132 display */

/* default is DOGM128_HW */
#if !defined DOGM128_HW && !defined DOGM132_HW && !defined DOGS102_HW
#define DOGM128_HW
#endif

// prevent duplicate definition
#if defined DOGM128_HW 
#if defined DOGM132_HW
#undef DOGM132_HW
#endif
#if defined DOGS102_HW
#undef DOGS102_HW
#endif
#endif
#if defined DOGM132_HW && defined DOGS102_HW
#undef DOGS102_HW
#endif


#include <stdint.h>
#include <avr/pgmspace.h>
#include "dogmfont.h"

#ifndef PGM_P
#define PGM_P const char *
#endif

#define PIN_SCK   13
#define PIN_MISO  12
#define PIN_MOSI  11
#define PIN_SS    10
#define PIN_A0_DEFAULT     6

#ifdef PROGMEM
#define dog_pgm_read_w(adr) pgm_read_word_near(adr)
#define dog_pgm_read(adr) pgm_read_byte_near(adr)
#else
#define dog_pgm_read_w(adr) (*(const int16_t *)(adr))
#define dog_pgm_read(adr) (*(const char *)(adr)) 
#endif

#ifndef PROGMEM
#define PROGMEM
#endif




/* hight of a page for all supported DOG devices */ 
#define DOG_PAGE_HEIGHT 8

/* setings for the various DOG displays */
#ifdef DOGM128_HW
#define DOG_WIDTH 128
#define DOG_HEIGHT 64
#endif

#ifdef DOGS102_HW
#define DOG_WIDTH 102
#define DOG_HEIGHT 64
#endif

#ifdef DOGM132_HW
#define DOG_WIDTH 132
#define DOG_HEIGHT 32
#endif

/* derived constants */
#define DOG_PAGE_CNT (DOG_HEIGHT/DOG_PAGE_HEIGHT)
#define DOG_PAGE_WIDTH DOG_WIDTH

#ifndef MACROS_H_
#define MACROS_H_


// from AVR035: Efficient C Coding for AVR

#define SETBIT(ADDRESS,BIT) (ADDRESS |= (1<<BIT))
#define CLEARBIT(ADDRESS,BIT) (ADDRESS &= ~(1<<BIT))
#define FLIPBIT(ADDRESS,BIT) (ADDRESS ^= (1<<BIT))
#define CHECKBIT(ADDRESS,BIT) (ADDRESS & (1<<BIT))

#define SETBITMASK(x,y) (x |= (y))
#define CLEARBITMASK(x,y) (x &= (~y))
#define FLIPBITMASK(x,y) (x ^= (y))
#define CHECKBITMASK(x,y) (x & (y))

#define VARFROMCOMB(x, y) x
#define BITFROMCOMB(x, y) y

#define C_SETBIT(comb) SETBIT(VARFROMCOMB(comb), BITFROMCOMB(comb))
#define C_CLEARBIT(comb) CLEARBIT(VARFROMCOMB(comb), BITFROMCOMB(comb))
#define C_FLIPBIT(comb) FLIPBIT(VARFROMCOMB(comb), BITFROMCOMB(comb))
#define C_CHECKBIT(comb) CHECKBIT(VARFROMCOMB(comb), BITFROMCOMB(comb))


#endif /* MACROS_H_ */

#define CS PORTB, 2
#define A0 PORTB, 1
#define RST PORTB, 0


#ifdef __cplusplus
extern "C" {
#endif


extern unsigned char dog_page_buffer[DOG_PAGE_WIDTH];

extern uint8_t dog_min_y;
extern uint8_t dog_max_y;

unsigned char dog_spi_out(unsigned char data);
void dog_spi_enable_client(void);
void dog_spi_disable_client(void);
void dog_cmd_mode(void);
void dog_data_mode(void);

  
void dog_Delay(uint16_t val);				/* delay in milliseconds */
void dog_Init(void);
void dog_SetContrast(uint8_t val);			/* values between 0 and 63 allowed, previosly named dog_set_contranst() */
void dog_SetInvertPixelMode(uint8_t val);	/* previosly named dog_set_inverse() */

  
  
/* --- page functions --- */

void dog_StartPage(void);
uint8_t dog_NextPage(void);

/* --- dogmspi.c --- */
extern uint8_t dog_spi_pin_a0;

void dog_spi_init(void);
unsigned char dog_spi_out(unsigned char data);
void dog_spi_enable_client(void);
void dog_spi_disable_client(void);
void dog_cmd_mode(void);
void dog_data_mode(void);


/* --- set/clr functions --- */

extern unsigned char dog_bit_to_mask[8];	/* dogmsd.c */
  
void dog_SetPixel(uint8_t x, uint8_t y);	/* dogmsp.c */
void dog_ClrPixel(uint8_t x, uint8_t y);	/* dogmcp.c */
void dog_XorPixel(uint8_t x, uint8_t y);	/* v1.01, dogmxp.c */

/* x1 must be lower or equal to x2 */
void dog_SetHLine(uint8_t x1, uint8_t x2, uint8_t y); 	/* v1.01, dogmsh.c */
void dog_ClrHLine(uint8_t x1, uint8_t x2, uint8_t y); 	/* v1.03, dogmch.c */
void dog_XorHLine(uint8_t x1, uint8_t x2, uint8_t y); 	/* v1.03, dogmxh.c */

uint8_t dog_get_vline_mask(uint8_t y1, uint8_t y2);	/* internal function, dogmov.c */
/* y1 must be lower or equal to y2 */
void dog_SetVLine(uint8_t x, uint8_t y1, uint8_t y2);			/* dogmsv.c */
void dog_ClrVLine(uint8_t x, uint8_t y1, uint8_t y2);			/* dogmcv.c */
void dog_XorVLine(uint8_t x, uint8_t y1, uint8_t y2);			/* dogmxv.c */

/* x1 must be lower or equal to x2 */
/* y1 must be lower or equal to y2 */
void dog_SetBox(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);	/* dogmsb.c */
void dog_ClrBox(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);	/* dogmcb.c */
void dog_XorBox(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);	/* dogmxb.c */

/* --- bitmap functions --- */

/* at position (x,y) set a pixel for each logical one bit in the bitmap pattern */
/* the bitmap must contain (w+7)/8 bytes, each byte is interpreted as bitmap pattern */
/* most significant bit of the byte in the pattern is on the left */
//void dog_SetHBitmap(uint8_t x, uint8_t y, const unsigned char *bitmap, uint8_t w); /* v1.01, dogmsm.c */
//void dog_SetHBitmapP(uint8_t x, uint8_t y, PGM_P bitmap, uint8_t w); /* v1.01, dogmsmp.c */
//void dog_SetBitmap(uint8_t x, uint8_t y, const unsigned char *bitmap, uint8_t w, uint8_t h); /* v1.01, dogmsm.c */
//void dog_SetBitmapP(uint8_t x, uint8_t y, PGM_P bitmap, uint8_t w, uint8_t h); /* v1.01, dogmsmp.c */

/* --- draw functions --- */
/*
  size = 0:
  +++
  +#+
  +++
  size = 1:
  +#+
  ###
  +#+
*/

void dog_DrawPoint(uint8_t x, uint8_t y, uint8_t size); 	/* dogmdp.c */
void dog_DrawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t size); 	/* dogmdl.c */
void dog_DrawArc(uint8_t mx, uint8_t my, uint8_t r, uint8_t w0, uint8_t w1, uint8_t size);  	/* dogmda.c */


/* --- font information --- */
uint8_t dog_GetCharWidth(PGM_P font, unsigned char code);	/* dogmfont.c */
uint8_t dog_GetStrWidth(PGM_P font, const char *s); /* dogmfont.c */

uint8_t dog_GetFontBBXHeight(PGM_P buf);
uint8_t dog_GetFontBBXWidth(PGM_P buf);
uint8_t dog_GetFontBBXDescent(PGM_P buf);
  
uint8_t dog_DrawChar(uint8_t x, uint8_t y, PGM_P font, unsigned char code); /* dogmfont.c */
uint8_t dog_DrawStr(uint8_t x, uint8_t y, PGM_P font, const char *s);	/* dogmfont.c */

/* --- math --- */

  /* w: 0..255     64 = PI/2, 128 = PI, 192 = 3*PI/2 */
  /* returns: -64 .. 64 */
signed char dog_sin(uint8_t w);	/* dogmma.c */
signed char dog_cos(uint8_t w);	/* dogmma.c */

/* --- other --- */
char *dog_itoa(unsigned long v);




#ifdef __cplusplus
}
#endif


#endif 

